@@ -4,6 +4,7 @@ from django import forms
 from django.core.exceptions import ValidationError
 from django.forms import inlineformset_factory
 from django.forms.models import modelformset_factory
+from django.utils import formats
 from django.utils.translation import ugettext_lazy as _
 from intervaltree import Interval, IntervalTree
 
@@ -11,10 +12,12 @@ from .constants import MIDNIGHT_TIME
 from .fields import FormTimeField, TimeStampDateField
 from .models import (
     ISO_WEEKDAY_CHOICES,
+    GeneralHolidaysHours,
     GeneralHolidaysTimeRange,
     OpeningHours,
     SpecificPeriodHours,
     SpecificPeriodTimeRange,
+    WeekDayHours,
     WeekDayTimeRange,
 )
 from .shortcuts import format_time_range, time_total_seconds
@@ -23,7 +26,7 @@ from .shortcuts import format_time_range, time_total_seconds
 class OpeningHoursModelForm(forms.ModelForm):
     class Meta:
         model = OpeningHours
-        fields = []
+        fields = ["always_open"]
 
 
 class TimeRangeModelForm(forms.ModelForm):
@@ -32,6 +35,11 @@ class TimeRangeModelForm(forms.ModelForm):
 
 
 class TimeRangeInlineFormSet(forms.models.BaseInlineFormSet):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        for form in self.forms:
+            form.empty_permitted = False
+
     def clean(self):
         super().clean()
 
@@ -46,36 +54,38 @@ class TimeRangeInlineFormSet(forms.models.BaseInlineFormSet):
 
         has_concurrent_periods = False
         for form in valid_forms:
-            opening_time = time_total_seconds(form.cleaned_data["opening_time"])
+            opening_time = form.cleaned_data.get("opening_time")
+            closing_time = form.cleaned_data.get("closing_time")
 
-            closing_time = form.cleaned_data["closing_time"]
+            if opening_time is not None and closing_time is not None:
+                opening_time = time_total_seconds(opening_time)
 
-            if closing_time == MIDNIGHT_TIME:
-                closing_time = time_total_seconds(time(23, 59)) + 1
-            else:
-                closing_time = time_total_seconds(closing_time)
+                if closing_time == MIDNIGHT_TIME:
+                    closing_time = time_total_seconds(time(23, 59)) + 1
+                else:
+                    closing_time = time_total_seconds(closing_time)
 
-            overlap_set = interval_tree.overlap(opening_time, closing_time)
+                overlap_set = interval_tree.overlap(opening_time, closing_time)
 
-            if len(overlap_set) > 0:
-                overlap = list(overlap_set)[0]
-                overlap_form_data = overlap.data.cleaned_data
-                overlap_opening_time = overlap_form_data["opening_time"]
-                overlap_closing_time = overlap_form_data["closing_time"]
-                overlap_display = format_time_range(
-                    overlap_opening_time, overlap_closing_time
-                )
+                if len(overlap_set) > 0:
+                    overlap = list(overlap_set)[0]
+                    overlap_form_data = overlap.data.cleaned_data
+                    overlap_opening_time = overlap_form_data["opening_time"]
+                    overlap_closing_time = overlap_form_data["closing_time"]
+                    overlap_display = format_time_range(
+                        overlap_opening_time, overlap_closing_time
+                    )
 
-                form.add_error(
-                    "opening_time",
-                    _(
-                        "This period is in conflict with the period {overlap_display}"
-                    ).format(overlap_display=overlap_display),
-                )
-                has_concurrent_periods = True
+                    form.add_error(
+                        "opening_time",
+                        _(
+                            "This period is in conflict with the period {overlap_display}"
+                        ).format(overlap_display=overlap_display),
+                    )
+                    has_concurrent_periods = True
 
-            interval = Interval(opening_time, closing_time, form)
-            interval_tree.add(interval)
+                interval = Interval(opening_time, closing_time, form)
+                interval_tree.add(interval)
 
         if has_concurrent_periods:
             raise ValidationError(
@@ -93,6 +103,12 @@ class WeekDayTimeRangeInlineFormSet(TimeRangeInlineFormSet):
     pass
 
 
+class WeekDayHoursModelForm(forms.ModelForm):
+    class Meta:
+        model = WeekDayHours
+        exclude = []
+
+
 class WeekDayHoursInlineFormSet(forms.models.BaseInlineFormSet):
     def __init__(self, *args, **kwargs):
         # Initialize week days only if no data is provided, otherwise no changes are detected on sub-forms save
@@ -120,6 +136,12 @@ class GeneralHolidaysHoursInlineFormSet(forms.models.BaseInlineFormSet):
         super().__init__(initial=initial, *args, **kwargs)
 
 
+class GeneralHolidaysHoursModelForm(forms.ModelForm):
+    class Meta:
+        model = GeneralHolidaysHours
+        exclude = []
+
+
 class SpecificPeriodTimeRangeModelForm(TimeRangeModelForm):
     class Meta:
         model = SpecificPeriodTimeRange
@@ -136,7 +158,7 @@ class SpecificPeriodHoursModelForm(forms.ModelForm):
 
     class Meta:
         model = SpecificPeriodHours
-        fields = ["from_date", "to_date", "closed"]
+        fields = ["from_date", "to_date", "closed", "closure_reason"]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -149,6 +171,11 @@ class SpecificPeriodHoursModelForm(forms.ModelForm):
 
 
 class SpecificPeriodHoursInlineFormSet(forms.models.BaseInlineFormSet):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        for form in self.forms:
+            form.empty_permitted = False
+
     def clean(self):
         super().clean()
 
@@ -178,8 +205,9 @@ class SpecificPeriodHoursInlineFormSet(forms.models.BaseInlineFormSet):
                     overlap_form_data = overlap.data.cleaned_data
                     overlap_from = overlap_form_data["from_date"]
                     overlap_to = overlap_form_data["to_date"]
+                    overlap_from = formats.date_format(overlap_from, "DATE_FORMAT")
+                    overlap_to = formats.date_format(overlap_to, "DATE_FORMAT")
                     overlap_display = f"{overlap_from} - {overlap_to}"
-
                     form.add_error(
                         "from_date",
                         _(
@@ -197,6 +225,36 @@ class SpecificPeriodHoursInlineFormSet(forms.models.BaseInlineFormSet):
             )
 
 
+WeekDayHoursFormSet = inlineformset_factory(
+    OpeningHours,
+    WeekDayHours,
+    form=WeekDayHoursModelForm,
+    formset=WeekDayHoursInlineFormSet,
+    can_delete=True,
+    extra=7,
+    max_num=7,
+)
+
+
+WeekDayHoursTimeRangeFormSet = inlineformset_factory(
+    WeekDayHours,
+    WeekDayTimeRange,
+    form=WeekDayTimeRangeModelForm,
+    formset=WeekDayTimeRangeInlineFormSet,
+    can_delete=True,
+    extra=0,
+)
+
+GeneralHolidaysTimeRangeFormSet = inlineformset_factory(
+    GeneralHolidaysHours,
+    GeneralHolidaysTimeRange,
+    form=GeneralHolidaysTimeRangeModelForm,
+    formset=GeneralHolidaysTimeRangeInlineFormSet,
+    can_delete=True,
+    extra=0,
+)
+
+
 SpecificPeriodHoursFormSet = inlineformset_factory(
     OpeningHours,
     SpecificPeriodHours,
